#include "pdc2.h"

/***************************************************************/
/* Mover started on 1/25/93. Most stuff is pretty straightforward. 
   Everything is done in x-y to make it nice and simple. R-theta
   would just be too much of a pain, and it would probably be more 
   incorrect because of the 1/r dependance.
*/

void move (int isp)
{
  register int rj,thj,n,j,i;
  register SCALAR at, art, temp, x2, y2;

  /* n = particle number
     w1,w2,w3,w4 are weights computed in gather, DUDE
     rj,thj are the lower grid position of the particle */

  /* djc changed start at 1 to start at 0 */
    for(i=0;i<ngr;i++)
      for(j=0;j<ngt;j++){
	ar[i][j] = a_scale[isp]*er[i][j];
	ath[i][j] = a_scale[isp]*eth[i][j];
      }

    for(n=np[isp]-1; n >= 0; n--) {
       /* rj & thj are ints, not SCALARs!! */
       /* when cent_cond, rj is offset by rminn */
       rj = (int) (r[isp][n]-rminn); /* rminn =0, if rmin = 0.0 */
       thj = (int)th[isp][n];

       if( (rj > 0)  || ( (rj == 0) && (cent_cond)) )
       {
         /* first find field at the particle */
         art  = ar[rj][thj]*w1[isp][n];
         art += ar[rj+1][thj]*w2[isp][n];
         art += ar[rj][thj+1]*w3[isp][n];
         art += ar[rj+1][thj+1]*w4[isp][n];
  
         at  = ath[rj][thj]*w1[isp][n];
         at += ath[rj+1][thj]*w2[isp][n];
         at += ath[rj][thj+1]*w3[isp][n];
         at += ath[rj+1][thj+1]*w4[isp][n];
       }

       else { /* rmin == 0.0 */
         /* if the particle is in the first cell do wierd things */
         art  = ar[0][thj]*(1.0-r[isp][n])*(1.0-th[isp][n] + thj);
         art += ar[1][thj]*r[isp][n]*(1.0-th[isp][n] + thj);
         art += ar[0][thj+1]*(1.0-r[isp][n])*(th[isp][n] - thj);
         art += ar[1][thj+1]*r[isp][n]*(th[isp][n] - thj);

         at  = ath[0][thj]*(1.0-r[isp][n])*(1.0-th[isp][n] + thj);
         at += ath[1][thj]*r[isp][n]*(1.0-th[isp][n] + thj);
         at += ath[0][thj+1]*(1.0-r[isp][n])*(th[isp][n] - thj);
         at += ath[1][thj+1]*r[isp][n]*(th[isp][n] - thj);

       }
       vr[isp][n] += art;          
       vth[isp][n] += at;

       x2 = r[isp][n] + vr[isp][n];
       y2 = vth[isp][n];
       if(x2!=0.0 && fabs(temp=(y2/x2))<max_ang) {
         r[isp][n]= x2*(2.0 + temp*temp)/2.0;
         if(x2 < (0.0)) {
           r[isp][n] = -r[isp][n];
           th[isp][n]+=nct/2 + temp/dth;
         }
         else th[isp][n]+= temp/dth;
       }
       else {
         r[isp][n]= sqrt(x2*x2 + y2*y2);
         th[isp][n]+= atan2(y2,x2)/dth;
       }
       if(th[isp][n] > nct) th[isp][n]-=nct;
       if(th[isp][n] < 0.0) th[isp][n]+=nct;
       if(r[isp][n]==0.0)  vth[isp][n]= 0.0;
       else {
          temp= (x2*vr[isp][n] +y2*vth[isp][n])/r[isp][n];
          vth[isp][n]= (-y2*vr[isp][n] +x2*vth[isp][n])/r[isp][n];
          vr[isp][n]= temp;
       }
/*        if (r[isp][n] < 0.0) printf ("PANIC %e %d %d\n",r[isp][n],isp,n); */

      if (cflag)
      {
       x[isp][n] = r[isp][n]*cos(th[isp][n]*dth);
       y[isp][n] = r[isp][n]*sin(th[isp][n]*dth);
       vx1[isp][n]=vr[isp][n]*cos(th[isp][n]*dth)
                  -vth[isp][n]*sin(th[isp][n]*dth);
       vy1[isp][n]=vr[isp][n]*sin(th[isp][n]*dth)
                  +vth[isp][n]*cos(th[isp][n]*dth);
      }
    }
}

void mag_move (int isp)
{
  register int rj,thj,n,j,i;
  register SCALAR at,art,vrtemp,vthtemp,vztemp,temp,x2,y2;

    for(i=0;i<ngr;i++)
      for(j=0;j<ngt;j++) {
        ar[i][j] = a_scale[isp]*er[i][j];
        ath[i][j] = a_scale[isp]*eth[i][j];
      }

    /* With Gyro-motion */
    for(n=np[isp]-1; n >= 0; n--) {
/*    if (gyro_flag[isp]) {		--------- not used -------------*/
	/* rj & thj are ints, not SCALARs!! */
        rj = (int) (r[isp][n]-rminn); /* rminn =0, if rmin = 0.0 */
	thj = (int)th[isp][n];

        if( (rj > 0)  || ( (rj == 0) && (cent_cond)) )
	{
	  /* first find field at the particle */
	  art  = ar[rj][thj]*w1[isp][n];
	  art += ar[rj+1][thj]*w2[isp][n];
	  art += ar[rj][thj+1]*w3[isp][n];
	  art += ar[rj+1][thj+1]*w4[isp][n];
	  
	  at  = ath[rj][thj]*w1[isp][n];
	  at += ath[rj+1][thj]*w2[isp][n];
	  at += ath[rj][thj+1]*w3[isp][n];
	  at += ath[rj+1][thj+1]*w4[isp][n];
	}

	else {		/* Without center conductor, at the origin */
	  /* if the particle is in the first cell do wierd things */

          art  = ar[0][thj]*(1.0-r[isp][n])*(1.0-th[isp][n] + thj);
          art += ar[1][thj]*r[isp][n]*(1.0-th[isp][n] + thj);
          art += ar[0][thj+1]*(1.0-r[isp][n])*(th[isp][n] - thj);
          art += ar[1][thj+1]*r[isp][n]*(th[isp][n] - thj);

          at  = ath[0][thj]*(1.0-r[isp][n])*(1.0-th[isp][n] + thj);
          at += ath[1][thj]*r[isp][n]*(1.0-th[isp][n] + thj);
          at += ath[0][thj+1]*(1.0-r[isp][n])*(th[isp][n] - thj);
          at += ath[1][thj+1]*r[isp][n]*(th[isp][n] - thj);

	}
        vrtemp = vr[isp][n] + art;
        vthtemp = vth[isp][n] + at;
        vztemp = vz[isp][n];
        
        vr[isp][n]= vrtemp + vthtemp*ttz[isp] - vztemp*tty[isp];
        vth[isp][n]= vthtemp + vztemp*ttx[isp] - vrtemp*ttz[isp];
        vz[isp][n]= vztemp + vrtemp*tty[isp] - vthtemp*ttx[isp];
        
        vrtemp += vth[isp][n]*ssz[isp] - vz[isp][n]*ssy[isp];
        vthtemp += vz[isp][n]*ssx[isp] - vr[isp][n]*ssz[isp];
        vztemp += vr[isp][n]*ssy[isp] - vth[isp][n]*ssx[isp];
        
        vr[isp][n] = vrtemp +art;
        vth[isp][n]= vthtemp+ at;
        vz[isp][n] = vztemp;

/*--------------------------------
      When we consider ONLY ExB drift without gyro-motion, very small time
      step should be used in order to avoid error from large dth per each dt.

      }		 * end of if(gyro...) * 
      else {		 * Without gyromotion, consider only ExB drift *
        rj = (int) (r[isp][n]-rminn);  * rminn =0, if rmin = 0.0 * 
        thj = (int)th[isp][n];
        vr[isp][n] = eth[rj][thj]*w1[isp][n] + eth[rj+1][thj]*w2[isp][n] +
                     eth[rj][thj+1]*w3[isp][n] + eth[rj+1][thj+1]*w4[isp][n];
        vth[isp][n] = er[rj][thj]*w1[isp][n] + er[rj+1][thj]*w2[isp][n] +
                     er[rj][thj+1]*w3[isp][n] + er[rj+1][thj+1]*w4[isp][n];
        vr[isp][n] /=vrscale[isp]*bmag;
        vth[isp][n]/=-vthscale[isp]*bmag;
      } 
-----------------------------------*/

      x2 = r[isp][n] + vr[isp][n];
      y2 = vth[isp][n];

      if(x2!=0.0 && fabs(temp=(y2/x2))<max_ang) {
        r[isp][n]= x2*(2.0 + temp*temp)/2.0;
        if(x2 < (0.0)) {
          r[isp][n] = -r[isp][n];
          th[isp][n]+=nct/2 + temp/dth;
        }
        else th[isp][n]+= temp/dth;
      }
      else {
        r[isp][n]= sqrt(x2*x2 + y2*y2);
        th[isp][n]+= atan2(y2,x2)/dth;
      }

      if(th[isp][n] > nct) th[isp][n]-=nct;
      if(th[isp][n] < 0.0) th[isp][n]+=nct;
      if(r[isp][n]==0.0) th[isp][n]= 0.0;
      else {
        temp= (x2*vr[isp][n] +y2*vth[isp][n])/r[isp][n];
        vth[isp][n]= (-y2*vr[isp][n] +x2*vth[isp][n])/r[isp][n];
        vr[isp][n]= temp;
      }
/*      if (r[isp][n] < 0.0) printf ("PANIC %e %d %d\n",r[isp][n],isp,n);*/

      if (cflag)
      {
        x[isp][n] = r[isp][n]*cos(th[isp][n]*dth);
        y[isp][n] = r[isp][n]*sin(th[isp][n]*dth);
        vx1[isp][n] = vr[isp][n]*cos(th[isp][n]*dth) 
                    - vth[isp][n]*sin(th[isp][n]*dth);
        vy1[isp][n] = vr[isp][n]*sin(th[isp][n]*dth) 
                    + vth[isp][n]*cos(th[isp][n]*dth);
      }
    } 	/* end of for (n=np[isp]...) */
}
