#include "pdc2.h"

void phase_calc();
int freqanalysis();
int realft(SCALAR *,int,int);
void Periodic_Smooth(SCALAR **, int, int);

/************************************************************************/
SCALAR revers(num,n)
int num, n;
{
  SCALAR power, rev;
  int inum, iquot, irem;
  
  rev = 0.;
  inum = num;
  power = 1.;
  
  do
  {
    iquot = inum/n;
    irem = inum - n*iquot;
    power /= n;
    rev += irem*power;
    inum = iquot;
  } while (inum > 0);
  
  return (rev);
}

/************************************************************************/
/* frand() returns values 0 through 1.                                  */
/* From "Random number generators: good ones are hard to find", S. Park */
/* and K. Miller, Communications of ACM, October 1988, pp 1192-1201.    */
/* This is from page 1195, and is to work on any system for which       */
/* maxint is 2**31-1 or larger. Due earlier to Schrage, as cited by P&M.*/
/*                                                                      */
/* Note: OK values for iseed are 1 through 2**31-2. Give it 0 or 2*31-1 */
/* and it will return the same values thereafter!                       */
/*                                                                      */
/* C version 6/91, Bruce Langdon.                                       */
/*                                                                      */
/* Algorithm replaces seed by mod(a*seed,m). First represent            */
/* seed = q*hi + lo.  Then                                              */
/* a*seed = a*q*hi + lo = (m - r)*hi + a*lo = (a*lo - r*hi) + m*hi,     */
/* and new seed = a*lo - r*hi unless negative; if so, then add m.       */

SCALAR frand()
{
  long a = 16807, m = 2147483647, q = 127773, r = 2836;
  long hi, lo;
  static long seed=31207321;
  SCALAR fnumb;

  hi = seed/q;
  lo = seed - q*hi;
  seed = a*lo - r*hi;
  /* "seed" will always be a legal integer of 32 bits (including sign). */
  if(seed <= 0) seed = seed + m;
  fnumb = seed/2147483646.0;

  return(fnumb);
}

/***********************************************************************/
/* time history accumulator; calculates and stores all history values, */
/* and performs combing on history values when low on memory           */

int history()
{
  register int i,j, k, isp, kk;
  static int count=1, nrftcount=0;
  SCALAR temp_energyx_hist, temp_energyy_hist, temp_energyz_hist, temp;

  /* imaginary component of DC value is not zero, 
     it is the value of the mode fftngt */
/*
  phase_calc();
*/

  for(i=0;i<ngr;i++) {
    fft_rho[i][0] = fabs(rhok[i][0]); 
    fft_phi[i][0] = fabs(phik[i][0]); 
    fft_rho[i][fftngt-1] = fabs(rhok[i][1]);
    fft_phi[i][fftngt-1] = fabs(phik[i][1]);
  }
  for(i=0;i<ngr;i++)
    for(j=1;j < fftngt-1 ; j++) {
      fft_rho[i][j] = sqrt(rhok[i][2*j]*rhok[i][2*j] + rhok[i][2*j+1]*rhok[i][2*j+1]);
      fft_phi[i][j] = sqrt(phik[i][2*j]*phik[i][2*j] + phik[i][2*j+1]*phik[i][2*j+1]);
    }

/*--------------------- 2D diagnostics -------------------------*/
  for (j=0;j<ngr;j++) {
    phi_r_show[j]=phi[j][0];
    er_r_show[j]=er[j][0];
    eth_r_show[j]=eth[j][0];
    for (i=0;i<nsp;i++) srho_r_show[i][j]=srho[i][j][0];
  }
/*--------------------- MCC rate show  -------------------------*/
  for(i=0;i<ndiag;i++) {
    for(j=0;j<ngr;j++) 
      for(k=0;k<ngt;k++) 
/*          rate_show[i][j][k]=mccrate[i][j][k]); */
        rate_show[i][j][k]=(rate_show[i][j][k]*(iter-1)+mccrate[i][j][k])/iter; 
    for(j=0;j<nsmooth; j++) Periodic_Smooth(rate_show[i], ncr, nct);
  }
/*--------------------------------------------------------------*/

  if(nfft) {
    if (thist_hi >= nfft) {
      freqanalysis();
      thist_hi=0;
    }
/*------------- not necessary
    for(k=0; k<num_kth; k++) {
      phi_r_kth_t[k][2*thist_hi]  = 0.0;
      phi_r_kth_t[k][2*thist_hi+1]= 0.0;  
    }
    for (i=ngr-7; i<(ngr-5); i++) {
        phi_r_kth_t[0][2*thist_hi]  += phik[i][0];
        phi_r_kth_t[0][2*thist_hi+1] += 0.0;
    } 
    for(k=1; k<num_kth; k++) {
      for (i=ngr-7; i<(ngr-5); i++) {
        phi_r_kth_t[k][2*thist_hi]  += phik[i][2*k];
        phi_r_kth_t[k][2*thist_hi+1] += phik[i][2*k+1];
      }
    } 
--------------*/
    cur_hist[thist_hi]= exti;
    phi_hist[0][thist_hi] = phi[0][0];
    phi_hist[1][thist_hi] = phi[ncr/2][0];
    phi_hist[2][thist_hi] = phi[ncr/2][nct/4];
    e_hist[0][thist_hi] = er[ncr/2][0];
    e_hist[1][thist_hi] = eth[ncr/2][0];
    Local_t_array[thist_hi] = t;
    thist_hi++;

    /********************************************/
   
    if(nrftcount == nfft)
    {

/* Time average quantities */

     for(i=0; i<ngr; i++)
      for(k=0; k<ngt; k++)
      {
        er_ave_show[i][k]= er_ave[i][k]/nfft;    er_ave[i][k]  = er[i][k];
        eth_ave_show[i][k]= eth_ave[i][k]/nfft;  eth_ave[i][k]  = eth[i][k];
        phi_ave_show[i][k]= phi_ave[i][k]/nfft;  phi_ave[i][k]= phi[i][k];
      }
      nrftcount =0;
    }
    else
    {
     for(i=0; i<ngr; i++)
      for(k=0;k<ngt;k++)
      {
        er_ave[i][k]   += er[i][k];
        eth_ave[i][k]   += eth[i][k];
        phi_ave[i][k] += phi[i][k];
      }
      nrftcount++;
    }

  }

  if (--count) return 0;                  /* only accum every interval steps */
  if (hist_hi >= HISTMAX) {              /* comb time histories */
    for (i=1, k=4; i<HISTMAX/4; i++, k+=4) {
      t_array[i] = t_array[k];
      dt_hist[i] = dt_hist[k];
      te_hist[i] = te_hist[k];
      ke_hist[i] = ke_hist[k];
      fe_hist[i] = fe_hist[k];
      fe_hist_old[i] = fe_hist_old[k];
      com_cur_hist[i] = com_cur_hist[k];
      sigma_hist[i] = sigma_hist[k];
      cem_hist[i] = cem_hist[k];
      a_momentum_hist[i] = a_momentum_hist[k];
      sigma_hist[i] = sigma_hist[k];
      for (j=0;j<3;j++) com_phi_hist[j][i] = com_phi_hist[j][k];
      for (j=0;j<2;j++) com_e_hist[j][i] = com_e_hist[j][k];
    }
    for (isp=0; isp<nsp; isp++) {
      for (i=1, k=4; i<HISTMAX/4; i++, k+=4) {
        np_hist[isp][i] = np_hist[isp][k];
        kes_hist[isp][i]= kes_hist[isp][k];
      }
    }
    for(j=0;j<num_modes;j++)
      for (i=1, k=4; i<HISTMAX/4; i++, k+=4) {
	fes_hist[j][i] = fes_hist[j][k];
        mode_hist[i]=mode_hist[k];
      }
    hist_hi = i;
    interval *= 4;  
  }
  t_array[hist_hi]  = t;                 /* accumulate histories */
  dt_hist[hist_hi]  = dt;                /* accumulate histories */
  com_cur_hist[hist_hi]= exti;
  sigma_hist[hist_hi] = sigma;
  ke_hist[hist_hi] = 0.0;
  cem_hist[hist_hi] = 0.0;
  a_momentum_hist[hist_hi] = 0.0;
  com_phi_hist[0][hist_hi] = phi[0][0];
  com_phi_hist[1][hist_hi] = phi[ncr/2][0];
  com_phi_hist[2][hist_hi] = phi[ncr/2][nct/16];
/* store E fields */
  com_e_hist[0][hist_hi] = er[ncr/2][0];
  com_e_hist[1][hist_hi] = eth[ncr/2][0];

  
  for (isp=0; isp<nsp; isp++) {
    np_hist[isp][hist_hi] = np[isp];
    temp_energyx_hist = temp_energyy_hist = temp_energyz_hist = 0.0;
    
    if (cflag) {
      for (i=0; i<np[isp]; i++) {
        temp_energyx_hist += vr[isp][i]*vr[isp][i];
        temp_energyy_hist += vth[isp][i]*vth[isp][i];
        temp_energyz_hist += vz[isp][i]*vz[isp][i];
      }
      temp_energyx_hist *= Exscale[isp];
      temp_energyy_hist *= Eyscale[isp];
      temp_energyz_hist *= Ezscale[isp];
      kes_hist[isp][hist_hi] = (temp_energyx_hist + temp_energyy_hist 
                              + temp_energyz_hist);
      ke_hist[hist_hi]     += kes_hist[isp][hist_hi];
      kes_hist[isp][hist_hi]= kes_hist[isp][hist_hi]+ 1e-30;
    }
  }
  if (cflag) {
    for (i=0; i<np[0]; i++) {
      cem_hist[hist_hi] += r[0][i];
      a_momentum_hist[hist_hi] += r[0][i]*vth[0][i];
    }
    if (np[0]) {
      cem_hist[hist_hi] /=np[0];
      a_momentum_hist[hist_hi]/=np[0];
    }
    else {
      cem_hist[hist_hi]=a_momentum_hist[hist_hi]=0;
    }
  }

  ke_hist[hist_hi] = ke_hist[hist_hi] +1e-30;           /* leehj */

  fe_hist[hist_hi] = fe_hist_old[hist_hi] = 0.0;
  for(k=0;k<nct;k++) {
    for(j=0;j<ngr;j++) {
      fe_hist[hist_hi]+=0.5*(er[j][k]*er[j][k]+eth[j][k]*eth[j][k])*
                        epsilon*vol[j]*drdthzlength;
      fe_hist_old[hist_hi]+=0.5*rho[j][k]*phi[j][k]*vol[j]*drdthzlength;
    }
  }

  te_hist[hist_hi]=fe_hist[hist_hi]+ke_hist[hist_hi];   /* leehj */

  if (cflag) {
    i = (int) moder;
    for(j=0,k=modsrt;j<num_modes;j++,k++) {
      fes_hist[j][hist_hi] = 0.5*(rhok[i][2*k]*phik[i][2*k]
				+rhok[i][2*k+1]*phik[i][2*k+1])*dr*zlength;
      fes_hist[j][hist_hi] = fabs(fes_hist[j][hist_hi]);
    }
    for(k=0;k<fftngt;k++) {
/*
      loc_ese[0][k]=fft_phi[ncr/4][k];
      loc_ese[1][k]=fft_phi[ncr/2][k];
      loc_ese[2][k]=fft_phi[3*ncr/4][k];
*/
      loc_ese[0][k]=fabs(rhok[ncr/4][2*k]*phik[ncr/4][2*k]+
                         rhok[ncr/4][2*k+1]*phik[ncr/4][2*k+1])*dr*zlength;
      loc_ese[1][k]=fabs(rhok[ncr/2][2*k]*phik[ncr/2][2*k]+
                         rhok[ncr/2][2*k+1]*phik[ncr/2][2*k+1])*dr*zlength;
      loc_ese[2][k]=fabs(rhok[3*ncr/4][2*k]*phik[3*ncr/4][2*k]+
                         rhok[3*ncr/4][2*k+1]*phik[3*ncr/4][2*k+1])*dr*zlength;
    }
/*    if (hist_hi%16==0)  
      for (k=0;k<num_modes2;k++) 
        mode_hist[k][mode_hist_hi=hist_hi/16]=loc_ese[1][k+modsrt];     */
    temp=0;
    for (k=1;k<fftngt/4;k++) 
    if (temp<loc_ese[1][k]) {
      temp=loc_ese[1][k];
      mode_hist[hist_hi]=k;
    }
  }
  hist_hi++;
  count = interval;

}

#ifdef COMMENTS
void phase_calc()
{
  static int i;
  static SCALAR tot[NBIN];
  int isp,n,num;
  SCALAR crossx,crossy,mag,phase;
  
  i++;
  for(isp=0;isp<nsp;isp++)
    for(n=np[isp]-1; n >= 0; n--) {
      /* find x and y components of v cross z hat (unit vector)*/
      mag = sqrt(vr[isp][n]*vr[isp][n]+vth[isp][n]*vth[isp][n]);
      crossx = vr[isp][n]/mag;
      crossy = vth[isp][n]/mag;
      phase = atan2(crossy,crossx+1e-12);
      if(phase<0.0)
	phase+=2*M_PI;
      num = (int) (phase * numphase / (2.0 * M_PI));
      tot[num]++;
      phasebin[num] = tot[num]/i;
    }
}
#endif

int err_end(error)
char *error;
{
  printf("%s\n",error);
  exit(-1);
}

/***************************************************************/
int freqanalysis()
{
  int i, j, k;
  SCALAR *temp1, *temp2;

  if(!(temp1= (SCALAR  *)malloc((nfft/2)*sizeof(SCALAR))))
  puts("Null ptr returned in freqanalysis()");
  if(!(temp2= (SCALAR  *)malloc((nfft/2)*sizeof(SCALAR))))
  puts("Null ptr returned in freqanalysis()");

/* Spectral component Calc */

  for(i=0; i< nfft; i++)
    {
      cur_fft[i] = cur_hist[i];
      phi_fft[i] = phi_hist[1][i];
      er_fft[i]  = e_hist[0][i];
      eth_fft[i]=  e_hist[1][i];
    }
 
  realft(phi_fft-1, freq_hi, 1);
  realft(cur_fft-1, freq_hi, 1);
  realft(er_fft-1, freq_hi, 1);
  realft(eth_fft-1, freq_hi, 1);

  /**** Computing mag and phase of the current signal ***/
  temp2[0]= (cur_fft[0] > 0.0) ? 0.0 : 180.0;
  temp1[0]= fabs(cur_fft[0])/freq_hi/2;
  for(i=1, j=2; i< freq_hi; i++, j+=2)
    {
      temp1[i]= sqrt(cur_fft[j]*cur_fft[j]+ cur_fft[j+1]*cur_fft[j+1])/freq_hi;
      if(fabs(cur_fft[j+1]) < 1e-30 && fabs(cur_fft[j]) < 1e-30)
        temp2[i]= 0.0;
      else
        temp2[i]= (-180./M_PI)*atan2(cur_fft[j+1], cur_fft[j]);
    }
  for(i=0; i< freq_hi; i++)
    {
       /* cur_fft[i]= log10(temp1[i] + DBL_MIN); */
      cur_fft[i]= temp1[i] ;
      cur_fft[freq_hi+i]= temp2[i];
    }

  /**** Computing mag and phase of the voltage signal ***/
  temp2[0]= (phi_fft[0] > 0.0) ? 0.0 : 180.0;
  temp1[0]= fabs(phi_fft[0])/freq_hi/2;
  for(i=1, j=2; i< freq_hi; i++, j+=2)
    {
      temp1[i]= sqrt(phi_fft[j]*phi_fft[j]+ phi_fft[j+1]*phi_fft[j+1])/freq_hi;
      if(fabs(phi_fft[j+1]) < 1e-30 && fabs(phi_fft[j]) < 1e-30)
        temp2[i]= 0.0;
      else
        temp2[i]= (-180./M_PI)*atan2(phi_fft[j+1], phi_fft[j]);
    }
  for(i=0; i< freq_hi; i++)
    {
      phi_fft[i]= temp1[i];
      phi_fft[freq_hi+i]= temp2[i];
    }

  /**** Computing mag and phase of the Er[ncr/2][0] signal ***/
  temp2[0]= (er_fft[0] > 0.0) ? 0.0 : 180.0;
  temp1[0]= fabs(er_fft[0])/freq_hi/2;
  for(i=1, j=2; i< freq_hi; i++, j+=2)
    {
      temp1[i]= sqrt(er_fft[j]*er_fft[j]+ er_fft[j+1]*er_fft[j+1])/freq_hi;
      if(fabs(er_fft[j+1]) < 1e-30 && fabs(er_fft[j]) < 1e-30)
        temp2[i]= 0.0;
      else
        temp2[i]= (-180./M_PI)*atan2(er_fft[j+1], er_fft[j]);
    }
  for(i=0; i< freq_hi; i++)
    {
      er_fft[i]= temp1[i];
      er_fft[freq_hi+i]= temp2[i];
    }


  /**** Computing mag and phase of the Eth[ncr/2][0] signal ***/
  temp2[0]= (eth_fft[0] > 0.0) ? 0.0 : 180.0;
  temp1[0]= fabs(eth_fft[0])/freq_hi/2;
  for(i=1, j=2; i< freq_hi; i++, j+=2)
    {
      temp1[i]= sqrt(eth_fft[j]*eth_fft[j]+ eth_fft[j+1]*eth_fft[j+1])/freq_hi;
      if(fabs(eth_fft[j+1]) < 1e-30 && fabs(eth_fft[j]) < 1e-30)
        temp2[i]= 0.0;
      else
        temp2[i]= (-180./M_PI)*atan2(eth_fft[j+1], eth_fft[j]);
    }
  for(i=0; i< freq_hi; i++)
    {
      eth_fft[i]= temp1[i];
      eth_fft[freq_hi+i]= temp2[i];
    }

/* end */

/*
  if (!cent_cond)
  {
    for(k=0; k<num_kth; k++)
    {
      for(i=0; i<2*nfft; i++)
      {
        temp2[i] = phi_r_kth_t[k][i];
      }                                           
      four1(temp2-1, nfft, 1);
    
      temp1[0]= temp2[0]*temp2[0] + temp2[1]*temp2[1];
      for(j=1; j< freq_hi; j++)
        temp1[j]= temp2[2*j]*temp2[2*j] + temp2[2*j+1]*temp2[2*j+1];
    }
  }
*/
  free(temp2);
  free(temp1);
}

/***************************************************************/
int history_nox()
{
  register int i,j, k, isp, kk;
  static int count=1, nrftcount=0;
  SCALAR temp_energyx_hist, temp_energyy_hist, temp_energyz_hist, temp;

  if (--count) return 0;                  /* only accum every interval steps */
  if (hist_hi >= HISTMAX) {              /* comb time histories */
    for (i=1, k=4; i<HISTMAX/4; i++, k+=4) {
      t_array[i] = t_array[k];
      ke_hist[i] = ke_hist[k];
      fe_hist[i] = fe_hist[k];
      com_phi_hist[1][i] = com_phi_hist[1][k];
    }
    for (isp=0; isp<nsp; isp++) {
      for (i=1, k=4; i<HISTMAX/4; i++, k+=4) {
        np_hist[isp][i] = np_hist[isp][k];
        kes_hist[isp][i]= kes_hist[isp][k];
      }
    }
    hist_hi = i;
    interval *= 4;
  }
  t_array[hist_hi]  = t;                 /* accumulate histories */
  com_phi_hist[1][hist_hi] = phi[ncr/2][0];
  ke_hist[hist_hi] = fe_hist[hist_hi] = 0.0;
  for(k=0;k<nct;k++) {
    for(j=0;j<ngr;j++) 
      fe_hist[hist_hi]+=0.5*(er[j][k]*er[j][k]+eth[j][k]*eth[j][k])*
                        epsilon*vol[j]*drdthzlength;
  }
  for (isp=0; isp<nsp; isp++) np_hist[isp][hist_hi] = np[isp];
  hist_hi++;
  count = interval;
}
